#include <vector>
#include <map>
#include <iostream>

struct Arc {
	char to;
	float weight;
};

constexpr float NOT_FOUND = 0.0F;

class Graph {
	public:
		void addArc(char from, char to, float weight = 1.0F) {
			auto& links = m_list[from];
			links.push_back({ .to = to, .weight = weight });
		}

		std::vector<char> getArcs(char from) {
			std::vector<char> arcs;
			for ( auto& arc : m_list.at(from) ) {
				arcs.push_back( arc.to );
			}
			return arcs;
		}

		float getWeight(char from, char to) {
			auto& links = m_list.at(from);
			for (auto& link : links) {
				if ( link.to == to ) {
					return link.weight;
				}
			}
			return NOT_FOUND;
		}

		inline bool hasVertex(char vertex) const {
			return m_list.find( vertex ) != m_list.end();
		}
	private:
		// there is a better structure we can use
		std::map<char, std::vector<Arc>> m_list;
};

int main( int argc, char* argv[] ) {
	Graph graph;
	graph.addArc('a', 'b');
	graph.addArc('b', 'a');

	std::cout << graph.getWeight('a', 'b') << std::endl;
	std::cout << graph.hasVertex('d') << std::endl;
	return 0;
}