#include <vector>
#include <iostream>
#include <fstream>
#include <string>

#include "labs/trees/trees.hpp"

static void build_static_tree() {
	Node* root = new Node();
	root->value = 5;

	// construct a tree
	add_node(root, 6);
	add_node(root, 3);
	add_node(root, 2);
	add_node(root, 1);
	add_node(root, -50);
	add_node(root, 10);

	// print out diagram
	std::ofstream diagram;
	diagram.open("tree.dia", std::ios::out);
	print_tree_dia(diagram, root);
	diagram.close();
}

static Node* build_tree_interactive() {
	// build phase
	std::string treeVal;
	Node* root = nullptr;

	while (treeVal != "done") {
		try {
			std::cin >> treeVal;
			if (treeVal == "done") {
				break;
			}

			int value = std::stoi(treeVal);
			if (root == nullptr) {
				root = new Node();
				root->value = value;
			}
			else {
				add_node(root, value);
			}
		}
		catch (std::invalid_argument&) {
			std::cout << "that is not a valid number" << std::endl;
		}
	}

	return root;
}

void search_tree_interactive(const Node* root) {
	// build phase
	std::string treeVal;

	std::cout << "search mode, enter number or done." << std::endl;
	while (treeVal != "done") {
		try {
			std::cin >> treeVal;
			if (treeVal == "done") {
				break;
			}

			int value = std::stoi(treeVal);
			bool result = search(root, value);
			if (result) {
				std::cout << value << " is located in the tree" << std::endl;
			} else {
				std::cout << value << " is *not* located in the tree" << std::endl;
			}
		}
		catch (std::invalid_argument&) {
			std::cout << "that is not a valid number" << std::endl;
		}
	}

}

static void interactive_tree() {
	std::cout << "TreeBuilder v1.0" << std::endl;
	std::cout << "type numbers, then done to ask questions" << std::endl;

	Node* root = build_tree_interactive();
	search_tree_interactive(root);
	delete root;
}

int main(int argc, const char* argv[]) {
	interactive_tree();
}
