#include <labs/trees/trees.hpp>
#include <gtest/gtest.h>

TEST(BinaryTree, TestAddBalanced) {
	Node* root = new Node();
	root->value = 42;

	add_node(root, 7);
	add_node(root, 100);

	EXPECT_NE(root->left, nullptr);
	EXPECT_NE(root->right, nullptr);

	ASSERT_EQ(7, root->left->value);
	ASSERT_EQ(100, root->right->value);
}

TEST(BinaryTree, TestAddUnbalancedLeft) {
	Node* root = new Node();
	root->value = 42;

	add_node(root, 7);
	add_node(root, 4);

	EXPECT_NE(root->left, nullptr);
	EXPECT_NE(root->left->left, nullptr);

	// right branches should be null
	EXPECT_EQ(root->right, nullptr);
	EXPECT_EQ(root->left->right, nullptr);

	ASSERT_EQ(7, root->left->value);
	ASSERT_EQ(4, root->left->left->value);
}

TEST(BinaryTree, TestAddUnbalancedRight) {
	Node* root = new Node();
	root->value = 42;

	add_node(root, 50);
	add_node(root, 100);

	// left nodes should be null
	EXPECT_EQ(root->left, nullptr);
	EXPECT_EQ(root->right->left, nullptr);

	// right branches should not be null
	EXPECT_NE(root->right, nullptr);
	EXPECT_NE(root->right->right, nullptr);

	ASSERT_EQ(50, root->right->value);
	ASSERT_EQ(100, root->right->right->value);
}

TEST(BinaryTree, TestAddUnbalancedRightLeft) {
	Node* root = new Node();
	root->value = 42;

	add_node(root, 50);
	add_node(root, 47);

	// nodes should be null
	EXPECT_EQ(root->left, nullptr);
	EXPECT_EQ(root->right->right, nullptr);

	// right branches should not be null
	EXPECT_NE(root->right, nullptr);
	EXPECT_NE(root->right->left, nullptr);

	ASSERT_EQ(50, root->right->value);
	ASSERT_EQ(47, root->right->left->value);
}

TEST(BinaryTree, TestAddUnbalancedLeftRight) {
	// TODO write a test case that places a value in root->left->right
	// you can use TestAddUnbalancedRightLeft to help you
}

#ifdef TREE_PART_2
//
// PART 2
// enable this by #define TREE_PART_2
// in the tree header
//

static Node* build_test_tree() {
	Node* root = new Node();
	root->value = 200;

	add_node(root, 290);
	add_node(root, 250);

	add_node(root, 110);
	add_node(root, 120);
	return root;
}

TEST(BinaryTree, TestSearchRoot) {
	Node* root = build_test_tree();
	ASSERT_EQ(true, search(root, 200));
	delete root;
}

TEST(BinaryTree, TestSearchRootMissing) {
	Node* root = build_test_tree();
	ASSERT_EQ(false, search(root, 310));
	delete root;
}

TEST(BinaryTree, TestSearchRootNull) {
	ASSERT_EQ(false, search(nullptr , 42));
}

TEST(BinaryTree, TestSearchLeft) {
	Node* root = build_test_tree();
	ASSERT_EQ(true, search(root, 110));
	delete root;
}

TEST(BinaryTree, TestSearchRight) {
	Node* root = build_test_tree();
	ASSERT_EQ(true, search(root, 290));
	delete root;
}
#endif

int main(int argc, char** argv) {
	::testing::InitGoogleTest(&argc, argv);
	return RUN_ALL_TESTS();
}